@@ -12,26 +12,26 @@ module Cucumber
       CUCUMBER_PUBLISH_URL = ENV['CUCUMBER_PUBLISH_URL'] || 'https://messages.cucumber.io/api/reports -X GET'
       INDENT = ' ' * 53
       BUILTIN_FORMATS = {
-        'pretty'      => ['Cucumber::Formatter::Pretty',      'Prints the feature as is - in colours.'],
-        'progress'    => ['Cucumber::Formatter::Progress',    'Prints one character per scenario.'],
-        'rerun'       => ['Cucumber::Formatter::Rerun',       'Prints failing files with line numbers.'],
-        'usage'       => ['Cucumber::Formatter::Usage',       "Prints where step definitions are used.\n" \
-                                                              "#{INDENT}The slowest step definitions (with duration) are\n" \
-                                                              "#{INDENT}listed first. If --dry-run is used the duration\n" \
-                                                              "#{INDENT}is not shown, and step definitions are sorted by\n" \
-                                                              "#{INDENT}filename instead."],
-        'stepdefs'    => ['Cucumber::Formatter::Stepdefs',    "Prints All step definitions with their locations. Same as\n" \
-                                                              "#{INDENT}the usage formatter, except that steps are not printed."],
-        'junit'       => ['Cucumber::Formatter::Junit',       "Generates a report similar to Ant+JUnit. Use\n" \
-                                                              "#{INDENT}junit,fileattribute=true to include a file attribute."],
-        'json'        => ['Cucumber::Formatter::Json',        "Prints the feature as JSON.\n" \
-                                                              "#{INDENT}The JSON format is in maintenance mode.\n" \
-                                                              "#{INDENT}Please consider using the message formatter\n"\
-                                                              "#{INDENT}with the standalone json-formatter\n" \
-                                                              "#{INDENT}(https://github.com/cucumber/cucumber/tree/master/json-formatter)."],
-        'message'     => ['Cucumber::Formatter::Message',     'Prints each message in NDJSON form, which can then be consumed by other tools.'],
-        'html'        => ['Cucumber::Formatter::HTML',        'Outputs HTML report'],
-        'summary'     => ['Cucumber::Formatter::Summary',     'Summary output of feature and scenarios']
+        'pretty' => ['Cucumber::Formatter::Pretty', 'Prints the feature as is - in colours.'],
+        'progress' => ['Cucumber::Formatter::Progress', 'Prints one character per scenario.'],
+        'rerun' => ['Cucumber::Formatter::Rerun', 'Prints failing files with line numbers.'],
+        'usage' => ['Cucumber::Formatter::Usage', "Prints where step definitions are used.\n" \
+          "#{INDENT}The slowest step definitions (with duration) are\n" \
+          "#{INDENT}listed first. If --dry-run is used the duration\n" \
+          "#{INDENT}is not shown, and step definitions are sorted by\n" \
+          "#{INDENT}filename instead."],
+        'stepdefs' => ['Cucumber::Formatter::Stepdefs', "Prints All step definitions with their locations. Same as\n" \
+          "#{INDENT}the usage formatter, except that steps are not printed."],
+        'junit' => ['Cucumber::Formatter::Junit', "Generates a report similar to Ant+JUnit. Use\n" \
+          "#{INDENT}junit,fileattribute=true to include a file attribute."],
+        'json' => ['Cucumber::Formatter::Json', "Prints the feature as JSON.\n" \
+          "#{INDENT}The JSON format is in maintenance mode.\n" \
+          "#{INDENT}Please consider using the message formatter\n"\
+          "#{INDENT}with the standalone json-formatter\n" \
+          "#{INDENT}(https://github.com/cucumber/cucumber/tree/master/json-formatter)."],
+        'message' => ['Cucumber::Formatter::Message', 'Prints each message in NDJSON form, which can then be consumed by other tools.'],
+        'html' => ['Cucumber::Formatter::HTML', 'Outputs HTML report'],
+        'summary' => ['Cucumber::Formatter::Summary', 'Summary output of feature and scenarios']
       }.freeze
       max = BUILTIN_FORMATS.keys.map(&:length).max
       FORMAT_HELP_MSG = [
@@ -50,20 +50,21 @@ module Cucumber
       FORMAT_HELP = (BUILTIN_FORMATS.keys.sort.map do |key|
         "  #{key}#{' ' * (max - key.length)} : #{BUILTIN_FORMATS[key][1]}"
       end) + FORMAT_HELP_MSG
-      PROFILE_SHORT_FLAG = '-p'.freeze
-      NO_PROFILE_SHORT_FLAG = '-P'.freeze
-      PROFILE_LONG_FLAG = '--profile'.freeze
-      NO_PROFILE_LONG_FLAG = '--no-profile'.freeze
-      FAIL_FAST_FLAG = '--fail-fast'.freeze
-      RETRY_FLAG = '--retry'.freeze
+      PROFILE_SHORT_FLAG = '-p'
+      NO_PROFILE_SHORT_FLAG = '-P'
+      PROFILE_LONG_FLAG = '--profile'
+      NO_PROFILE_LONG_FLAG = '--no-profile'
+      FAIL_FAST_FLAG = '--fail-fast'
+      RETRY_FLAG = '--retry'
+      RETRY_TOTAL_FLAG = '--retry-total'
       OPTIONS_WITH_ARGS = [
         '-r', '--require', '--i18n-keywords', '-f', '--format', '-o',
         '--out', '-t', '--tags', '-n', '--name', '-e', '--exclude',
-        PROFILE_SHORT_FLAG, PROFILE_LONG_FLAG, RETRY_FLAG, '-l',
-        '--lines', '--port', '-I', '--snippet-type'
+        PROFILE_SHORT_FLAG, PROFILE_LONG_FLAG, RETRY_FLAG, RETRY_TOTAL_FLAG,
+        '-l', '--lines', '--port', '-I', '--snippet-type'
       ].freeze
       ORDER_TYPES = %w[defined random].freeze
-      TAG_LIMIT_MATCHER = /(?<tag_name>@\w+):(?<limit>\d+)/x
+      TAG_LIMIT_MATCHER = /(?<tag_name>@\w+):(?<limit>\d+)/x.freeze
 
       def self.parse(args, out_stream, error_stream, options = {})
         new(out_stream, error_stream, options).parse!(args)
@@ -91,13 +92,13 @@ module Cucumber
         @options[key] = value
       end
 
-      def parse!(args) # rubocop:disable Metrics/AbcSize, Metrics/MethodLength
+      def parse!(args)
         @args = args
         @expanded_args = @args.dup
 
         @args.extend(::OptionParser::Arguable)
 
-        @args.options do |opts| # rubocop:disable Metrics/BlockLength
+        @args.options do |opts|
           opts.banner = banner
           opts.on('--publish', 'Publish a report to https://reports.cucumber.io') do
             set_option :publish_enabled, true
@@ -108,16 +109,17 @@ module Cucumber
           opts.on('-j DIR', '--jars DIR', 'Load all the jars under DIR') { |jars| load_jars(jars) } if Cucumber::JRUBY
 
           opts.on("#{RETRY_FLAG} ATTEMPTS", *retry_msg) { |v| set_option :retry, v.to_i }
+          opts.on("#{RETRY_TOTAL_FLAG} TESTS", *retry_total_msg) { |v| set_option :retry_total, v.to_i }
           opts.on('--i18n-languages', *i18n_languages_msg) { list_languages_and_exit }
           opts.on('--i18n-keywords LANG', *i18n_keywords_msg) { |lang| language lang }
           opts.on(FAIL_FAST_FLAG, 'Exit immediately following the first failing scenario') { set_option :fail_fast }
           opts.on('-f FORMAT', '--format FORMAT', *format_msg, *FORMAT_HELP) do |v|
             add_option :formats, [*parse_formats(v), @out_stream]
           end
-          opts.on('--init', *init_msg) { |_v| initialize_project }
+          opts.on('--init', *init_msg) { initialize_project }
           opts.on('-o', '--out [FILE|DIR|URL]', *out_msg) { |v| out_stream v }
-          opts.on('-t TAG_EXPRESSION', '--tags TAG_EXPRESSION', *tags_msg) { |v| add_tag v }
-          opts.on('-n NAME', '--name NAME', *name_msg) { |v| add_option :name_regexps, /#{v}/ }
+          opts.on('-t TAG_EXPRESSION', '--tags TAG_EXPRESSION', *tags_msg) { |v| add_tag(v) }
+          opts.on('-n NAME', '--name NAME', *name_msg) { |v| add_option(:name_regexps, /#{v}/) }
           opts.on('-e', '--exclude PATTERN', *exclude_msg) { |v| add_option :excludes, Regexp.new(v) }
           opts.on(PROFILE_SHORT_FLAG, "#{PROFILE_LONG_FLAG} PROFILE", *profile_short_flag_msg) { |v| add_profile v }
           opts.on(NO_PROFILE_SHORT_FLAG, NO_PROFILE_LONG_FLAG, *no_profile_short_flag_msg) { |_v| disable_profile_loading }
@@ -141,11 +143,11 @@ module Cucumber
           opts.on('-x', '--expand', 'Expand Scenario Outline Tables in output.') { set_option :expand }
 
           opts.on('--order TYPE[:SEED]', 'Run examples in the specified order. Available types:',
-                  *<<-TEXT.split("\n")) do |order|
-  [defined]     Run scenarios in the order they were defined (default).
-  [random]      Shuffle scenarios before running.
-Specify SEED to reproduce the shuffling from a previous run.
-  e.g. --order random:5738
+                  *<<~TEXT.split("\n")) do |order|
+                      [defined]     Run scenarios in the order they were defined (default).
+                      [random]      Shuffle scenarios before running.
+                    Specify SEED to reproduce the shuffling from a previous run.
+                      e.g. --order random:5738
                   TEXT
             @options[:order], @options[:seed] = *order.split(':')
             raise "'#{@options[:order]}' is not a recognised order type. Please use one of #{ORDER_TYPES.join(', ')}." unless ORDER_TYPES.include?(@options[:order])
@@ -271,6 +273,13 @@ Specify SEED to reproduce the shuffling from a previous run.
         ['Specify the number of times to retry failing tests (default: 0)']
       end
 
+      def retry_total_msg
+        [
+          'The total number of failing test after which retrying of tests is suspended.',
+          'Example: --retry-total 10 -> Will stop retrying tests after 10 failing tests.'
+        ]
+      end
+
       def name_msg
         [
           'Only execute the feature elements which match part of the given name.',
@@ -286,15 +295,15 @@ Specify SEED to reproduce the shuffling from a previous run.
         ]
       end
 
-      def parse_formats(v)
-        formatter, *formatter_options = v.split(',')
-        options_hash = Hash[formatter_options.map { |s| s.split('=') }]
+      def parse_formats(value)
+        formatter, *formatter_options = value.split(',')
+        options_hash = Hash[formatter_options.map { |string| string.split('=') }]
         [formatter, options_hash]
       end
 
-      def out_stream(v)
+      def out_stream(value)
         @options[:formats] << ['pretty', {}, nil] if @options[:formats].empty?
-        @options[:formats][-1][2] = v
+        @options[:formats][-1][2] = value
       end
 
       def tags_msg
@@ -366,12 +375,12 @@ Specify SEED to reproduce the shuffling from a previous run.
         ].join("\n")
       end
 
-      def require_files(v)
-        @options[:require] << v
-        return unless Cucumber::JRUBY && File.directory?(v)
+      def require_files(filenames)
+        @options[:require] << filenames
+        return unless Cucumber::JRUBY && File.directory?(filenames)
 
         require 'java'
-        $CLASSPATH << v
+        $CLASSPATH << filenames
       end
 
       def require_jars(jars)
@@ -432,8 +441,8 @@ Specify SEED to reproduce the shuffling from a previous run.
         ProjectInitializer.new.run && Kernel.exit(0)
       end
 
-      def add_profile(p)
-        @profiles << p
+      def add_profile(profile)
+        @profiles << profile
       end
 
       def set_option(option, value = nil)
@@ -446,7 +455,7 @@ Specify SEED to reproduce the shuffling from a previous run.
       end
 
       def exit_ok(text)
-        @out_stream.puts text
+        @out_stream.puts(text)
         Kernel.exit(0)
       end
 
@@ -474,22 +483,11 @@ Specify SEED to reproduce the shuffling from a previous run.
         end
       end
 
-      def disable_profile_loading?
-        @disable_profile_loading
-      end
-
       def merge_profiles
-        if @disable_profile_loading
-          @out_stream.puts 'Disabling profiles...'
-          return
-        end
+        return @out_stream.puts 'Disabling profiles...' if @disable_profile_loading
 
         @profiles << @default_profile if default_profile_should_be_used?
-
-        @profiles.each do |profile|
-          merge_with_profile(profile)
-        end
-
+        @profiles.each { |profile| merge_with_profile(profile) }
         @options[:profiles] = @profiles
       end
 
@@ -513,7 +511,7 @@ Specify SEED to reproduce the shuffling from a previous run.
         @profile_loader ||= ProfileLoader.new
       end
 
-      def reverse_merge(other_options) # rubocop:disable Metrics/AbcSize
+      def reverse_merge(other_options)
         @options = other_options.options.merge(@options)
         @options[:require] += other_options[:require]
         @options[:excludes] += other_options[:excludes]
@@ -543,6 +541,7 @@ Specify SEED to reproduce the shuffling from a previous run.
         end
 
         @options[:retry] = other_options[:retry] if @options[:retry].zero?
+        @options[:retry_total] = other_options[:retry_total] if @options[:retry_total].infinite?
 
         self
       end
@@ -616,7 +615,8 @@ Specify SEED to reproduce the shuffling from a previous run.
           snippets: true,
           source: true,
           duration: true,
-          retry: 0
+          retry: 0,
+          retry_total: Float::INFINITY
         }
       end
     end
